/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2013, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.appendlog;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayDeque;

/**
 * // TODO: Document this
 *
 * @author Radim Vansa &lt;rvansa@redhat.com&gt;
 */
public class WaitingAppendLog extends AbstractAppendLog {
   private final ArrayDeque<ByteBuffer> queue = new ArrayDeque<ByteBuffer>(100);
   private final int limit;
   private final long waitTime;

   private long pushIndex = 0;
   private long popIndex = 0;

   public WaitingAppendLog(String filename, long waitTime, int limit) throws FileNotFoundException {
      super(filename);
      this.waitTime = waitTime;
      this.limit = limit;
   }

   @Override
   public void append(ByteBuffer bytes) throws IOException {
      synchronized (fileChannel) {
         queue.push(bytes);
         pushIndex++;
         long myIndex = pushIndex;
         if (queue.size() < limit) {
            try {
               fileChannel.wait(waitTime);
            } catch (InterruptedException e) {
            }
         }
         // if the buffer for this thread has been written down, do not write further buffers
         // without this the threads would be convoying, not reducing number of flushes (much)
         if (myIndex > popIndex) {
            do {
               fileChannel.write(queue.pop());
               popIndex++;
            } while (!queue.isEmpty());
            flush();
            fileChannel.notifyAll();
         }
      }
   }

   @Override
   public void close() throws IOException {
      synchronized (fileChannel) {
         if (!queue.isEmpty()) {
            throw new IllegalStateException();
         }
         fileChannel.close();
      }
   }

   @Override
   public String toString() {
      return "WaitingAppendLog{" +
            "limit=" + limit +
            ", waitTime=" + waitTime +
            '}';
   }
}
